@@ -9,11 +9,11 @@ async def main():
 async def producer(send_channel):
     async with send_channel:
         for i in range(3):
-            await send_channel.send("message {}".format(i))
+            await send_channel.send(f"message {i}")
 
 async def consumer(receive_channel):
     async with receive_channel:
         async for value in receive_channel:
-            print("got value {!r}".format(value))
+            print(f"got value {value!r}")
 
 trio.run(main)
