@@ -13,19 +13,19 @@ async def sender(client_stream):
     print("sender: started!")
     while True:
         data = b"async can sometimes be confusing, but I believe in you!"
-        print("sender: sending {!r}".format(data))
+        print(f"sender: sending {data!r}")
         await client_stream.send_all(data)
         await trio.sleep(1)
 
 async def receiver(client_stream):
     print("receiver: started!")
     async for data in client_stream:
-        print("receiver: got data {!r}".format(data))
+        print(f"receiver: got data {data!r}")
     print("receiver: connection closed")
     sys.exit()
 
 async def parent():
-    print("parent: connecting to 127.0.0.1:{}".format(PORT))
+    print(f"parent: connecting to 127.0.0.1:{PORT}")
     client_stream = await trio.open_tcp_stream("127.0.0.1", PORT)
     async with client_stream:
         async with trio.open_nursery() as nursery:
