@@ -5,7 +5,7 @@ from trio._util import NoPublicConstructor
 
 class TrioInternalError(Exception):
     """Raised by :func:`run` if we encounter a bug in Trio, or (possibly) a
-    misuse of one of the low-level :mod:`trio.hazmat` APIs.
+    misuse of one of the low-level :mod:`trio.lowlevel` APIs.
 
     This should never happen! If you get this error, please file a bug.
 
@@ -25,9 +25,7 @@ class RunFinishedError(RuntimeError):
 
 
 class WouldBlock(Exception):
-    """Raised by ``X_nowait`` functions if ``X`` would block.
-
-    """
+    """Raised by ``X_nowait`` functions if ``X`` would block."""
 
 
 class Cancelled(BaseException, metaclass=NoPublicConstructor):
@@ -62,6 +60,7 @@ class Cancelled(BaseException, metaclass=NoPublicConstructor):
        everywhere.
 
     """
+
     def __str__(self):
         return "Cancelled"
 
