@@ -2,12 +2,15 @@ import itertools
 from contextlib import contextmanager
 import enum
 import socket
+import sys
+from typing import TYPE_CHECKING
 
-import outcome
 import attr
+from outcome import Value
 
 from .. import _core
 from ._run import _public
+from ._io_common import wake_all
 
 from ._windows_cffi import (
     ffi,
@@ -25,6 +28,8 @@ from ._windows_cffi import (
     IoControlCodes,
 )
 
+assert not TYPE_CHECKING or sys.platform == "win32"
+
 # There's a lot to be said about the overall design of a Windows event
 # loop. See
 #
@@ -171,7 +176,8 @@ class CKeys(enum.IntEnum):
     AFD_POLL = 0
     WAIT_OVERLAPPED = 1
     LATE_CANCEL = 2
-    USER_DEFINED = 3  # and above
+    FORCE_WAKEUP = 3
+    USER_DEFINED = 4  # and above
 
 
 def _check(success):
@@ -180,7 +186,7 @@ def _check(success):
     return success
 
 
-def _get_base_socket(sock, *, which=WSAIoctls.SIO_BASE_HANDLE):
+def _get_underlying_socket(sock, *, which=WSAIoctls.SIO_BASE_HANDLE):
     if hasattr(sock, "fileno"):
         sock = sock.fileno()
     base_ptr = ffi.new("HANDLE *")
@@ -202,6 +208,53 @@ def _get_base_socket(sock, *, which=WSAIoctls.SIO_BASE_HANDLE):
     return base_ptr[0]
 
 
+def _get_base_socket(sock):
+    # There is a development kit for LSPs called Komodia Redirector.
+    # It does some unusual (some might say evil) things like intercepting
+    # SIO_BASE_HANDLE (fails) and SIO_BSP_HANDLE_SELECT (returns the same
+    # socket) in a misguided attempt to prevent bypassing it. It's been used
+    # in malware including the infamous Lenovo Superfish incident from 2015,
+    # but unfortunately is also used in some legitimate products such as
+    # parental control tools and Astrill VPN. Komodia happens to not
+    # block SIO_BSP_HANDLE_POLL, so we'll try SIO_BASE_HANDLE and fall back
+    # to SIO_BSP_HANDLE_POLL if it doesn't work.
+    # References:
+    # - https://github.com/piscisaureus/wepoll/blob/0598a791bf9cbbf480793d778930fc635b044980/wepoll.c#L2223
+    # - https://github.com/tokio-rs/mio/issues/1314
+
+    while True:
+        try:
+            # If this is not a Komodia-intercepted socket, we can just use
+            # SIO_BASE_HANDLE.
+            return _get_underlying_socket(sock)
+        except OSError as ex:
+            if ex.winerror == ErrorCodes.ERROR_NOT_SOCKET:
+                # SIO_BASE_HANDLE might fail even without LSP intervention,
+                # if we get something that's not a socket.
+                raise
+            if hasattr(sock, "fileno"):
+                sock = sock.fileno()
+            sock = _handle(sock)
+            next_sock = _get_underlying_socket(
+                sock, which=WSAIoctls.SIO_BSP_HANDLE_POLL
+            )
+            if next_sock == sock:
+                # If BSP_HANDLE_POLL returns the same socket we already had,
+                # then there's no layering going on and we need to fail
+                # to prevent an infinite loop.
+                raise RuntimeError(
+                    "Unexpected network configuration detected: "
+                    "SIO_BASE_HANDLE failed and SIO_BSP_HANDLE_POLL didn't "
+                    "return a different socket. Please file a bug at "
+                    "https://github.com/python-trio/trio/issues/new, "
+                    "and include the output of running: "
+                    "netsh winsock show catalog"
+                )
+            # Otherwise we've gotten at least one layer deeper, so
+            # loop back around to keep digging.
+            sock = next_sock
+
+
 def _afd_helper_handle():
     # The "AFD" driver is exposed at the NT path "\Device\Afd". We're using
     # the Win32 CreateFile, though, so we have to pass a Win32 path. \\.\ is
@@ -264,13 +317,14 @@ WRITABLE_FLAGS = (
 )
 
 
-# Annoyingly, while the API makes it *seem* like you can happily issue
-# as many independent AFD_POLL operations as you want without them interfering
-# with each other, in fact if you issue two AFD_POLL operations for the same
-# socket at the same time, then Windows gets super confused. For example, if
-# we issue one operation from wait_readable, and another independent operation
-# from wait_writable, then Windows may complete the wait_writable operation
-# when the socket becomes readable.
+# Annoyingly, while the API makes it *seem* like you can happily issue as many
+# independent AFD_POLL operations as you want without them interfering with
+# each other, in fact if you issue two AFD_POLL operations for the same socket
+# at the same time with notification going to the same IOCP port, then Windows
+# gets super confused. For example, if we issue one operation from
+# wait_readable, and another independent operation from wait_writable, then
+# Windows may complete the wait_writable operation when the socket becomes
+# readable.
 #
 # To avoid this, we have to coalesce all the operations on a single socket
 # into one, and when the set of waiters changes we have to throw away the old
@@ -290,6 +344,24 @@ class AFDPollOp:
     lpOverlapped = attr.ib()
     poll_info = attr.ib()
     waiters = attr.ib()
+    afd_group = attr.ib()
+
+
+# The Windows kernel has a weird issue when using AFD handles. If you have N
+# instances of wait_readable/wait_writable registered with a single AFD handle,
+# then cancelling any one of them takes something like O(N**2) time. So if we
+# used just a single AFD handle, then cancellation would quickly become very
+# expensive, e.g. a program with N active sockets would take something like
+# O(N**3) time to unwind after control-C. The solution is to spread our sockets
+# out over multiple AFD handles, so that N doesn't grow too large for any
+# individual handle.
+MAX_AFD_GROUP_SIZE = 500  # at 1000, the cubic scaling is just starting to bite
+
+
+@attr.s(slots=True, eq=False)
+class AFDGroup:
+    size = attr.ib()
+    handle = attr.ib()
 
 
 @attr.s(slots=True, eq=False, frozen=True)
@@ -321,17 +393,14 @@ class WindowsIOManager:
         # touches to safe values up front, before we do anything that can
         # fail.
         self._iocp = None
-        self._afd = None
+        self._all_afd_handles = []
 
         self._iocp = _check(
-            kernel32.CreateIoCompletionPort(
-                INVALID_HANDLE_VALUE, ffi.NULL, 0, 0
-            )
+            kernel32.CreateIoCompletionPort(INVALID_HANDLE_VALUE, ffi.NULL, 0, 0)
         )
         self._events = ffi.new("OVERLAPPED_ENTRY[]", MAX_EVENTS)
 
-        self._afd = _afd_helper_handle()
-        self._register_with_iocp(self._afd, CKeys.AFD_POLL)
+        self._vacant_afd_groups = set()
         # {lpOverlapped: AFDPollOp}
         self._afd_ops = {}
         # {socket handle: AFDWaiters}
@@ -345,21 +414,41 @@ class WindowsIOManager:
         self._completion_key_counter = itertools.count(CKeys.USER_DEFINED)
 
         with socket.socket() as s:
-            # LSPs can't override this.
-            base_handle = _get_base_socket(s, which=WSAIoctls.SIO_BASE_HANDLE)
+            # We assume we're not working with any LSP that changes
+            # how select() is supposed to work. Validate this by
+            # ensuring that the result of SIO_BSP_HANDLE_SELECT (the
+            # LSP-hookable mechanism for "what should I use for
+            # select()?") matches that of SIO_BASE_HANDLE ("what is
+            # the real non-hooked underlying socket here?").
+            #
+            # This doesn't work for Komodia-based LSPs; see the comments
+            # in _get_base_socket() for details. But we have special
+            # logic for those, so we just skip this check if
+            # SIO_BASE_HANDLE fails.
+
             # LSPs can in theory override this, but we believe that it never
-            # actually happens in the wild.
-            select_handle = _get_base_socket(
+            # actually happens in the wild (except Komodia)
+            select_handle = _get_underlying_socket(
                 s, which=WSAIoctls.SIO_BSP_HANDLE_SELECT
             )
-            if base_handle != select_handle:  # pragma: no cover
-                raise RuntimeError(
-                    "Unexpected network configuration detected. "
-                    "Please file a bug at "
-                    "https://github.com/python-trio/trio/issues/new, "
-                    "and include the output of running: "
-                    "netsh winsock show catalog"
-                )
+            try:
+                # LSPs shouldn't override this...
+                base_handle = _get_underlying_socket(s, which=WSAIoctls.SIO_BASE_HANDLE)
+            except OSError:
+                # But Komodia-based LSPs do anyway, in a way that causes
+                # a failure with WSAEFAULT. We have special handling for
+                # them in _get_base_socket(). Make sure it works.
+                _get_base_socket(s)
+            else:
+                if base_handle != select_handle:
+                    raise RuntimeError(
+                        "Unexpected network configuration detected: "
+                        "SIO_BASE_HANDLE and SIO_BSP_HANDLE_SELECT differ. "
+                        "Please file a bug at "
+                        "https://github.com/python-trio/trio/issues/new, "
+                        "and include the output of running: "
+                        "netsh winsock show catalog"
+                    )
 
     def close(self):
         try:
@@ -368,10 +457,9 @@ class WindowsIOManager:
                 self._iocp = None
                 _check(kernel32.CloseHandle(iocp))
         finally:
-            if self._afd is not None:
-                afd = self._afd
-                self._afd = None
-                _check(kernel32.CloseHandle(afd))
+            while self._all_afd_handles:
+                afd_handle = self._all_afd_handles.pop()
+                _check(kernel32.CloseHandle(afd_handle))
 
     def __del__(self):
         self.close()
@@ -391,7 +479,14 @@ class WindowsIOManager:
             completion_key_monitors=len(self._completion_key_queues),
         )
 
-    def handle_io(self, timeout):
+    def force_wakeup(self):
+        _check(
+            kernel32.PostQueuedCompletionStatus(
+                self._iocp, 0, CKeys.FORCE_WAKEUP, ffi.NULL
+            )
+        )
+
+    def get_events(self, timeout):
         received = ffi.new("PULONG")
         milliseconds = round(1000 * timeout)
         if timeout > 0 and milliseconds == 0:
@@ -399,15 +494,17 @@ class WindowsIOManager:
         try:
             _check(
                 kernel32.GetQueuedCompletionStatusEx(
-                    self._iocp, self._events, MAX_EVENTS, received,
-                    milliseconds, 0
+                    self._iocp, self._events, MAX_EVENTS, received, milliseconds, 0
                 )
             )
         except OSError as exc:
             if exc.winerror != ErrorCodes.WAIT_TIMEOUT:  # pragma: no cover
                 raise
-            return
-        for i in range(received[0]):
+            return 0
+        return received[0]
+
+    def process_events(self, received):
+        for i in range(received):
             entry = self._events[i]
             if entry.lpCompletionKey == CKeys.AFD_POLL:
                 lpo = entry.lpOverlapped
@@ -434,7 +531,12 @@ class WindowsIOManager:
             elif entry.lpCompletionKey == CKeys.WAIT_OVERLAPPED:
                 # Regular I/O event, dispatch on lpOverlapped
                 waiter = self._overlapped_waiters.pop(entry.lpOverlapped)
-                _core.reschedule(waiter)
+                overlapped = entry.lpOverlapped
+                transferred = entry.dwNumberOfBytesTransferred
+                info = CompletionKeyEventInfo(
+                    lpOverlapped=overlapped, dwNumberOfBytesTransferred=transferred
+                )
+                _core.reschedule(waiter, Value(info))
             elif entry.lpCompletionKey == CKeys.LATE_CANCEL:
                 # Post made by a regular I/O event's abort_fn
                 # after it failed to cancel the I/O. If we still
@@ -469,24 +571,21 @@ class WindowsIOManager:
                     # try changing this line to
                     # _core.reschedule(waiter, outcome.Error(exc))
                     raise exc
+            elif entry.lpCompletionKey == CKeys.FORCE_WAKEUP:
+                pass
             else:
                 # dispatch on lpCompletionKey
                 queue = self._completion_key_queues[entry.lpCompletionKey]
                 overlapped = int(ffi.cast("uintptr_t", entry.lpOverlapped))
                 transferred = entry.dwNumberOfBytesTransferred
                 info = CompletionKeyEventInfo(
-                    lpOverlapped=overlapped,
-                    dwNumberOfBytesTransferred=transferred,
+                    lpOverlapped=overlapped, dwNumberOfBytesTransferred=transferred
                 )
                 queue.put_nowait(info)
 
     def _register_with_iocp(self, handle, completion_key):
         handle = _handle(handle)
-        _check(
-            kernel32.CreateIoCompletionPort(
-                handle, self._iocp, completion_key, 0
-            )
-        )
+        _check(kernel32.CreateIoCompletionPort(handle, self._iocp, completion_key, 0))
         # Supposedly this makes things slightly faster, by disabling the
         # ability to do WaitForSingleObject(handle). We would never want to do
         # that anyway, so might as well get the extra speed (if any).
@@ -504,16 +603,21 @@ class WindowsIOManager:
     def _refresh_afd(self, base_handle):
         waiters = self._afd_waiters[base_handle]
         if waiters.current_op is not None:
+            afd_group = waiters.current_op.afd_group
             try:
                 _check(
                     kernel32.CancelIoEx(
-                        self._afd, waiters.current_op.lpOverlapped
+                        afd_group.handle, waiters.current_op.lpOverlapped
                     )
                 )
             except OSError as exc:
-                if exc.winerror != ErrorCodes.ERROR_NOT_FOUND:  # pragma: no cover
-                    raise
+                if exc.winerror != ErrorCodes.ERROR_NOT_FOUND:
+                    # I don't think this is possible, so if it happens let's
+                    # crash noisily.
+                    raise  # pragma: no cover
             waiters.current_op = None
+            afd_group.size -= 1
+            self._vacant_afd_groups.add(afd_group)
 
         flags = 0
         if waiters.read_task is not None:
@@ -524,6 +628,14 @@ class WindowsIOManager:
         if not flags:
             del self._afd_waiters[base_handle]
         else:
+            try:
+                afd_group = self._vacant_afd_groups.pop()
+            except KeyError:
+                afd_group = AFDGroup(0, _afd_helper_handle())
+                self._register_with_iocp(afd_group.handle, CKeys.AFD_POLL)
+                self._all_afd_handles.append(afd_group.handle)
+            self._vacant_afd_groups.add(afd_group)
+
             lpOverlapped = ffi.new("LPOVERLAPPED")
 
             poll_info = ffi.new("AFD_POLL_INFO *")
@@ -537,7 +649,7 @@ class WindowsIOManager:
             try:
                 _check(
                     kernel32.DeviceIoControl(
-                        self._afd,
+                        afd_group.handle,
                         IoControlCodes.IOCTL_AFD_POLL,
                         poll_info,
                         ffi.sizeof("AFD_POLL_INFO"),
@@ -548,12 +660,21 @@ class WindowsIOManager:
                     )
                 )
             except OSError as exc:
-                if exc.winerror != ErrorCodes.ERROR_IO_PENDING:  # pragma: no cover
-                    raise
-
-            op = AFDPollOp(lpOverlapped, poll_info, waiters)
+                if exc.winerror != ErrorCodes.ERROR_IO_PENDING:
+                    # This could happen if the socket handle got closed behind
+                    # our back while a wait_* call was pending, and we tried
+                    # to re-issue the call. Clear our state and wake up any
+                    # pending calls.
+                    del self._afd_waiters[base_handle]
+                    # Do this last, because it could raise.
+                    wake_all(waiters, exc)
+                    return
+            op = AFDPollOp(lpOverlapped, poll_info, waiters, afd_group)
             waiters.current_op = op
             self._afd_ops[lpOverlapped] = op
+            afd_group.size += 1
+            if afd_group.size >= MAX_AFD_GROUP_SIZE:
+                self._vacant_afd_groups.remove(afd_group)
 
     async def _afd_poll(self, sock, mode):
         base_handle = _get_base_socket(sock)
@@ -564,6 +685,8 @@ class WindowsIOManager:
         if getattr(waiters, mode) is not None:
             raise _core.BusyResourceError
         setattr(waiters, mode, _core.current_task())
+        # Could potentially raise if the handle is somehow invalid; that's OK,
+        # we let it escape.
         self._refresh_afd(base_handle)
 
         def abort_fn(_):
@@ -586,18 +709,7 @@ class WindowsIOManager:
         handle = _get_base_socket(handle)
         waiters = self._afd_waiters.get(handle)
         if waiters is not None:
-            if waiters.read_task is not None:
-                _core.reschedule(
-                    waiters.read_task,
-                    outcome.Error(_core.ClosedResourceError())
-                )
-                waiters.read_task = None
-            if waiters.write_task is not None:
-                _core.reschedule(
-                    waiters.write_task,
-                    outcome.Error(_core.ClosedResourceError())
-                )
-                waiters.write_task = None
+            wake_all(waiters, _core.ClosedResourceError())
             self._refresh_afd(handle)
 
     ################################################################
@@ -656,7 +768,7 @@ class WindowsIOManager:
                     ) from exc
             return _core.Abort.FAILED
 
-        await _core.wait_task_rescheduled(abort)
+        info = await _core.wait_task_rescheduled(abort)
         if lpOverlapped.Internal != 0:
             # the lpOverlapped reports the error as an NT status code,
             # which we must convert back to a Win32 error code before
@@ -669,11 +781,10 @@ class WindowsIOManager:
                     # We didn't request this cancellation, so assume
                     # it happened due to the underlying handle being
                     # closed before the operation could complete.
-                    raise _core.ClosedResourceError(
-                        "another task closed this resource"
-                    )
+                    raise _core.ClosedResourceError("another task closed this resource")
             else:
                 raise_winerror(code)
+        return info
 
     async def _perform_overlapped(self, handle, submit_fn):
         # submit_fn(lpOverlapped) submits some I/O
@@ -700,7 +811,7 @@ class WindowsIOManager:
             def submit_write(lpOverlapped):
                 # yes, these are the real documented names
                 offset_fields = lpOverlapped.DUMMYUNIONNAME.DUMMYSTRUCTNAME
-                offset_fields.Offset = file_offset & 0xffffffff
+                offset_fields.Offset = file_offset & 0xFFFFFFFF
                 offset_fields.OffsetHigh = file_offset >> 32
                 _check(
                     kernel32.WriteFile(
@@ -722,7 +833,7 @@ class WindowsIOManager:
 
             def submit_read(lpOverlapped):
                 offset_fields = lpOverlapped.DUMMYUNIONNAME.DUMMYSTRUCTNAME
-                offset_fields.Offset = file_offset & 0xffffffff
+                offset_fields.Offset = file_offset & 0xFFFFFFFF
                 offset_fields.OffsetHigh = file_offset >> 32
                 _check(
                     kernel32.ReadFile(
