@@ -3,6 +3,8 @@ import attr
 import trio
 from .abc import HalfCloseableStream
 
+from trio._util import Final
+
 
 async def aclose_forcefully(resource):
     """Close an async resource or async generator immediately, without
@@ -35,7 +37,7 @@ async def aclose_forcefully(resource):
 
 
 @attr.s(eq=False, hash=False)
-class StapledStream(HalfCloseableStream):
+class StapledStream(HalfCloseableStream, metaclass=Final):
     """This class `staples <https://en.wikipedia.org/wiki/Staple_(fastener)>`__
     together two unidirectional streams to make single bidirectional stream.
 
@@ -69,19 +71,16 @@ class StapledStream(HalfCloseableStream):
        is delegated to this object.
 
     """
+
     send_stream = attr.ib()
     receive_stream = attr.ib()
 
     async def send_all(self, data):
-        """Calls ``self.send_stream.send_all``.
-
-        """
+        """Calls ``self.send_stream.send_all``."""
         return await self.send_stream.send_all(data)
 
     async def wait_send_all_might_not_block(self):
-        """Calls ``self.send_stream.wait_send_all_might_not_block``.
-
-        """
+        """Calls ``self.send_stream.wait_send_all_might_not_block``."""
         return await self.send_stream.wait_send_all_might_not_block()
 
     async def send_eof(self):
@@ -97,15 +96,11 @@ class StapledStream(HalfCloseableStream):
             return await self.send_stream.aclose()
 
     async def receive_some(self, max_bytes=None):
-        """Calls ``self.receive_stream.receive_some``.
-
-        """
+        """Calls ``self.receive_stream.receive_some``."""
         return await self.receive_stream.receive_some(max_bytes)
 
     async def aclose(self):
-        """Calls ``aclose`` on both underlying streams.
-
-        """
+        """Calls ``aclose`` on both underlying streams."""
         try:
             await self.send_stream.aclose()
         finally:
