@@ -1,13 +1,13 @@
-import os as _os
-import sys as _sys
+import os
+import sys
 import select
 import socket as _stdlib_socket
 from functools import wraps as _wraps
+from typing import TYPE_CHECKING
 
 import idna as _idna
 
 import trio
-from ._util import fspath
 from . import _core
 
 
@@ -30,7 +30,7 @@ class _try_sync:
             return self._blocking_exc_override(exc)
 
     async def __aenter__(self):
-        await trio.hazmat.checkpoint_if_cancelled()
+        await trio.lowlevel.checkpoint_if_cancelled()
 
     async def __aexit__(self, etype, value, tb):
         if value is not None and self._is_blocking_io_error(value):
@@ -38,7 +38,7 @@ class _try_sync:
             # block
             return True
         else:
-            await trio.hazmat.cancel_shielded_checkpoint()
+            await trio.lowlevel.cancel_shielded_checkpoint()
             # Let the return or exception propagate
             return False
 
@@ -50,9 +50,9 @@ class _try_sync:
 try:
     from socket import IPPROTO_IPV6
 except ImportError:
-    # As of at least 3.6, python on Windows is missing IPPROTO_IPV6
+    # Before Python 3.8, Windows is missing IPPROTO_IPV6
     # https://bugs.python.org/issue29515
-    if _sys.platform == "win32":
+    if sys.platform == "win32":  # pragma: no branch
         IPPROTO_IPV6 = 41
 
 ################################################################
@@ -151,8 +151,10 @@ async def getaddrinfo(host, port, family=0, type=0, proto=0, flags=0):
     # with the _NUMERIC_ONLY flags set, and then only spawn a thread if that
     # fails with EAI_NONAME:
     def numeric_only_failure(exc):
-        return isinstance(exc, _stdlib_socket.gaierror) and \
-            exc.errno == _stdlib_socket.EAI_NONAME
+        return (
+            isinstance(exc, _stdlib_socket.gaierror)
+            and exc.errno == _stdlib_socket.EAI_NONAME
+        )
 
     async with _try_sync(numeric_only_failure):
         return _stdlib_socket.getaddrinfo(
@@ -186,7 +188,7 @@ async def getaddrinfo(host, port, family=0, type=0, proto=0, flags=0):
             type,
             proto,
             flags,
-            cancellable=True
+            cancellable=True,
         )
 
 
@@ -229,7 +231,7 @@ async def getprotobyname(name):
 
 
 def from_stdlib_socket(sock):
-    """Convert a standard library :func:`socket.socket` object into a Trio
+    """Convert a standard library :class:`socket.socket` object into a Trio
     socket object.
 
     """
@@ -238,14 +240,14 @@ def from_stdlib_socket(sock):
 
 @_wraps(_stdlib_socket.fromfd, assigned=(), updated=())
 def fromfd(fd, family, type, proto=0):
-    """Like :func:`socket.fromfd`, but returns a Trio socket object.
-
-    """
+    """Like :func:`socket.fromfd`, but returns a Trio socket object."""
     family, type, proto = _sniff_sockopts_for_fileno(family, type, proto, fd)
     return from_stdlib_socket(_stdlib_socket.fromfd(fd, family, type, proto))
 
 
-if hasattr(_stdlib_socket, "fromshare"):
+if sys.platform == "win32" or (
+    not TYPE_CHECKING and hasattr(_stdlib_socket, "fromshare")
+):
 
     @_wraps(_stdlib_socket.fromshare, assigned=(), updated=())
     def fromshare(*args, **kwargs):
@@ -267,9 +269,9 @@ def socket(
     family=_stdlib_socket.AF_INET,
     type=_stdlib_socket.SOCK_STREAM,
     proto=0,
-    fileno=None
+    fileno=None,
 ):
-    """Create a new Trio socket, like :func:`socket.socket`.
+    """Create a new Trio socket, like :class:`socket.socket`.
 
     This function's behavior can be customized using
     :func:`set_custom_socket_factory`.
@@ -280,29 +282,20 @@ def socket(
         if sf is not None:
             return sf.socket(family, type, proto)
     else:
-        family, type, proto = _sniff_sockopts_for_fileno(
-            family, type, proto, fileno
-        )
+        family, type, proto = _sniff_sockopts_for_fileno(family, type, proto, fileno)
     stdlib_socket = _stdlib_socket.socket(family, type, proto, fileno)
     return from_stdlib_socket(stdlib_socket)
 
 
 def _sniff_sockopts_for_fileno(family, type, proto, fileno):
-    """Correct SOCKOPTS for given fileno, falling back to provided values.
-
-    """
+    """Correct SOCKOPTS for given fileno, falling back to provided values."""
     # Wrap the raw fileno into a Python socket object
     # This object might have the wrong metadata, but it lets us easily call getsockopt
     # and then we'll throw it away and construct a new one with the correct metadata.
-    if not _sys.platform == "linux":
+    if sys.platform != "linux":
         return family, type, proto
-    try:
-        from socket import SO_DOMAIN, SO_PROTOCOL
-    except ImportError:
-        # Only available on 3.6 and above:
-        SO_PROTOCOL = 38
-        SO_DOMAIN = 39
-    from socket import SOL_SOCKET, SO_TYPE
+    from socket import SO_DOMAIN, SO_PROTOCOL, SOL_SOCKET, SO_TYPE
+
     sockobj = _stdlib_socket.socket(family, type, proto, fileno=fileno)
     try:
         family = sockobj.getsockopt(SOL_SOCKET, SO_DOMAIN)
@@ -331,10 +324,8 @@ _SOCK_TYPE_MASK = ~(
 )
 
 
-# Note that this is *not* in __all__.
-#
 # This function will modify the given socket to match the behavior in python
-# 3.7. This will become unecessary and can be removed when support for versions
+# 3.7. This will become unnecessary and can be removed when support for versions
 # older than 3.7 is dropped.
 def real_socket_type(type_num):
     return type_num & _SOCK_TYPE_MASK
@@ -347,19 +338,94 @@ def _make_simple_sock_method_wrapper(methname, wait_fn, maybe_avail=False):
     async def wrapper(self, *args, **kwargs):
         return await self._nonblocking_helper(fn, args, kwargs, wait_fn)
 
-    wrapper.__doc__ = (
-        """Like :meth:`socket.socket.{}`, but async.
+    wrapper.__doc__ = f"""Like :meth:`socket.socket.{methname}`, but async.
 
-            """.format(methname)
-    )
+            """
     if maybe_avail:
         wrapper.__doc__ += (
-            "Only available on platforms where :meth:`socket.socket.{}` "
-            "is available.".format(methname)
+            f"Only available on platforms where :meth:`socket.socket.{methname}` is "
+            "available."
         )
     return wrapper
 
 
+# Helpers to work with the (hostname, port) language that Python uses for socket
+# addresses everywhere. Split out into a standalone function so it can be reused by
+# FakeNet.
+
+# Take an address in Python's representation, and returns a new address in
+# the same representation, but with names resolved to numbers,
+# etc.
+#
+# local=True means that the address is being used with bind() or similar
+# local=False means that the address is being used with connect() or sendto() or
+# similar.
+#
+# NOTE: this function does not always checkpoint
+async def _resolve_address_nocp(type, family, proto, *, ipv6_v6only, address, local):
+    # Do some pre-checking (or exit early for non-IP sockets)
+    if family == _stdlib_socket.AF_INET:
+        if not isinstance(address, tuple) or not len(address) == 2:
+            raise ValueError("address should be a (host, port) tuple")
+    elif family == _stdlib_socket.AF_INET6:
+        if not isinstance(address, tuple) or not 2 <= len(address) <= 4:
+            raise ValueError(
+                "address should be a (host, port, [flowinfo, [scopeid]]) tuple"
+            )
+    elif family == _stdlib_socket.AF_UNIX:
+        # unwrap path-likes
+        return os.fspath(address)
+    else:
+        return address
+
+    # -- From here on we know we have IPv4 or IPV6 --
+    host, port, *_ = address
+    # Fast path for the simple case: already-resolved IP address,
+    # already-resolved port. This is particularly important for UDP, since
+    # every sendto call goes through here.
+    if isinstance(port, int):
+        try:
+            _stdlib_socket.inet_pton(family, address[0])
+        except (OSError, TypeError):
+            pass
+        else:
+            return address
+    # Special cases to match the stdlib, see gh-277
+    if host == "":
+        host = None
+    if host == "<broadcast>":
+        host = "255.255.255.255"
+    flags = 0
+    if local:
+        flags |= _stdlib_socket.AI_PASSIVE
+    # Since we always pass in an explicit family here, AI_ADDRCONFIG
+    # doesn't add any value -- if we have no ipv6 connectivity and are
+    # working with an ipv6 socket, then things will break soon enough! And
+    # if we do enable it, then it makes it impossible to even run tests
+    # for ipv6 address resolution on travis-ci, which as of 2017-03-07 has
+    # no ipv6.
+    # flags |= AI_ADDRCONFIG
+    if family == _stdlib_socket.AF_INET6 and not ipv6_v6only:
+        flags |= _stdlib_socket.AI_V4MAPPED
+    gai_res = await getaddrinfo(host, port, family, type, proto, flags)
+    # AFAICT from the spec it's not possible for getaddrinfo to return an
+    # empty list.
+    assert len(gai_res) >= 1
+    # Address is the last item in the first entry
+    (*_, normed), *_ = gai_res
+    # The above ignored any flowid and scopeid in the passed-in address,
+    # so restore them if present:
+    if family == _stdlib_socket.AF_INET6:
+        normed = list(normed)
+        assert len(normed) == 4
+        if len(address) >= 3:
+            normed[2] = address[2]
+        if len(address) >= 4:
+            normed[3] = address[3]
+        normed = tuple(normed)
+    return normed
+
+
 class SocketType:
     def __init__(self):
         raise TypeError(
@@ -446,21 +512,20 @@ class _SocketType(SocketType):
         return repr(self._sock).replace("socket.socket", "trio.socket.socket")
 
     def dup(self):
-        """Same as :meth:`socket.socket.dup`.
-
-        """
+        """Same as :meth:`socket.socket.dup`."""
         return _SocketType(self._sock.dup())
 
     def close(self):
         if self._sock.fileno() != -1:
-            trio.hazmat.notify_closing(self._sock)
+            trio.lowlevel.notify_closing(self._sock)
             self._sock.close()
 
     async def bind(self, address):
-        address = await self._resolve_local_address(address)
+        address = await self._resolve_address_nocp(address, local=True)
         if (
             hasattr(_stdlib_socket, "AF_UNIX")
-            and self.family == _stdlib_socket.AF_UNIX and address[0]
+            and self.family == _stdlib_socket.AF_UNIX
+            and address[0]
         ):
             # Use a thread for the filesystem traversal (unless it's an
             # abstract domain socket)
@@ -470,6 +535,7 @@ class _SocketType(SocketType):
             # complete asynchronously, like connect. But in practice AFAICT
             # there aren't yet any real systems that do this, so we'll worry
             # about it when it happens.
+            await trio.lowlevel.checkpoint()
             return self._sock.bind(address)
 
     def shutdown(self, flag):
@@ -481,7 +547,7 @@ class _SocketType(SocketType):
 
     def is_readable(self):
         # use select.select on Windows, and select.poll everywhere else
-        if _sys.platform == "win32":
+        if sys.platform == "win32":
             rready, _, _ = select.select([self._sock], [], [], 0)
             return bool(rready)
         p = select.poll()
@@ -491,77 +557,21 @@ class _SocketType(SocketType):
     async def wait_writable(self):
         await _core.wait_writable(self._sock)
 
-    ################################################################
-    # Address handling
-    ################################################################
-
-    # Take an address in Python's representation, and returns a new address in
-    # the same representation, but with names resolved to numbers,
-    # etc.
-    async def _resolve_address(self, address, flags):
-        # Do some pre-checking (or exit early for non-IP sockets)
-        if self._sock.family == _stdlib_socket.AF_INET:
-            if not isinstance(address, tuple) or not len(address) == 2:
-                raise ValueError("address should be a (host, port) tuple")
-        elif self._sock.family == _stdlib_socket.AF_INET6:
-            if not isinstance(address, tuple) or not 2 <= len(address) <= 4:
-                raise ValueError(
-                    "address should be a (host, port, [flowinfo, [scopeid]]) "
-                    "tuple"
-                )
-        elif self._sock.family == _stdlib_socket.AF_UNIX:
-            await trio.hazmat.checkpoint()
-            # unwrap path-likes
-            return fspath(address)
-
-        else:
-            await trio.hazmat.checkpoint()
-            return address
-        host, port, *_ = address
-        # Special cases to match the stdlib, see gh-277
-        if host == "":
-            host = None
-        if host == "<broadcast>":
-            host = "255.255.255.255"
-        # Since we always pass in an explicit family here, AI_ADDRCONFIG
-        # doesn't add any value -- if we have no ipv6 connectivity and are
-        # working with an ipv6 socket, then things will break soon enough! And
-        # if we do enable it, then it makes it impossible to even run tests
-        # for ipv6 address resolution on travis-ci, which as of 2017-03-07 has
-        # no ipv6.
-        # flags |= AI_ADDRCONFIG
-        if self._sock.family == _stdlib_socket.AF_INET6:
-            if not self._sock.getsockopt(
+    async def _resolve_address_nocp(self, address, *, local):
+        if self.family == _stdlib_socket.AF_INET6:
+            ipv6_v6only = self._sock.getsockopt(
                 IPPROTO_IPV6, _stdlib_socket.IPV6_V6ONLY
-            ):
-                flags |= _stdlib_socket.AI_V4MAPPED
-        gai_res = await getaddrinfo(
-            host, port, self._sock.family, self.type, self._sock.proto, flags
+            )
+        else:
+            ipv6_v6only = False
+        return await _resolve_address_nocp(
+            self.type,
+            self.family,
+            self.proto,
+            ipv6_v6only=ipv6_v6only,
+            address=address,
+            local=local,
         )
-        # AFAICT from the spec it's not possible for getaddrinfo to return an
-        # empty list.
-        assert len(gai_res) >= 1
-        # Address is the last item in the first entry
-        (*_, normed), *_ = gai_res
-        # The above ignored any flowid and scopeid in the passed-in address,
-        # so restore them if present:
-        if self._sock.family == _stdlib_socket.AF_INET6:
-            normed = list(normed)
-            assert len(normed) == 4
-            if len(address) >= 3:
-                normed[2] = address[2]
-            if len(address) >= 4:
-                normed[3] = address[3]
-            normed = tuple(normed)
-        return normed
-
-    # Returns something appropriate to pass to bind()
-    async def _resolve_local_address(self, address):
-        return await self._resolve_address(address, _stdlib_socket.AI_PASSIVE)
-
-    # Returns something appropriate to pass to connect()/sendto()/sendmsg()
-    async def _resolve_remote_address(self, address):
-        return await self._resolve_address(address, 0)
 
     async def _nonblocking_helper(self, fn, args, kwargs, wait_fn):
         # We have to reconcile two conflicting goals:
@@ -602,9 +612,7 @@ class _SocketType(SocketType):
     _accept = _make_simple_sock_method_wrapper("accept", _core.wait_readable)
 
     async def accept(self):
-        """Like :meth:`socket.socket.accept`, but async.
-
-        """
+        """Like :meth:`socket.socket.accept`, but async."""
         sock, addr = await self._accept()
         return from_stdlib_socket(sock), addr
 
@@ -618,7 +626,7 @@ class _SocketType(SocketType):
         # notification. This means it isn't really cancellable... we close the
         # socket if cancelled, to avoid confusion.
         try:
-            address = await self._resolve_remote_address(address)
+            address = await self._resolve_address_nocp(address, local=False)
             async with _try_sync():
                 # An interesting puzzle: can a non-blocking connect() return EINTR
                 # (= raise InterruptedError)? PEP 475 specifically left this as
@@ -678,11 +686,9 @@ class _SocketType(SocketType):
             self._sock.close()
             raise
         # Okay, the connect finished, but it might have failed:
-        err = self._sock.getsockopt(
-            _stdlib_socket.SOL_SOCKET, _stdlib_socket.SO_ERROR
-        )
+        err = self._sock.getsockopt(_stdlib_socket.SOL_SOCKET, _stdlib_socket.SO_ERROR)
         if err != 0:
-            raise OSError(err, "Error in connect: " + _os.strerror(err))
+            raise OSError(err, "Error in connect: " + os.strerror(err))
 
     ################################################################
     # recv
@@ -694,17 +700,13 @@ class _SocketType(SocketType):
     # recv_into
     ################################################################
 
-    recv_into = _make_simple_sock_method_wrapper(
-        "recv_into", _core.wait_readable
-    )
+    recv_into = _make_simple_sock_method_wrapper("recv_into", _core.wait_readable)
 
     ################################################################
     # recvfrom
     ################################################################
 
-    recvfrom = _make_simple_sock_method_wrapper(
-        "recvfrom", _core.wait_readable
-    )
+    recvfrom = _make_simple_sock_method_wrapper("recvfrom", _core.wait_readable)
 
     ################################################################
     # recvfrom_into
@@ -744,13 +746,11 @@ class _SocketType(SocketType):
 
     @_wraps(_stdlib_socket.socket.sendto, assigned=(), updated=())
     async def sendto(self, *args):
-        """Similar to :meth:`socket.socket.sendto`, but async.
-
-        """
+        """Similar to :meth:`socket.socket.sendto`, but async."""
         # args is: data[, flags], address)
         # and kwargs are not accepted
         args = list(args)
-        args[-1] = await self._resolve_remote_address(args[-1])
+        args[-1] = await self._resolve_address_nocp(args[-1], local=False)
         return await self._nonblocking_helper(
             _stdlib_socket.socket.sendto, args, {}, _core.wait_writable
         )
@@ -759,7 +759,9 @@ class _SocketType(SocketType):
     # sendmsg
     ################################################################
 
-    if hasattr(_stdlib_socket.socket, "sendmsg"):
+    if sys.platform != "win32" or (
+        not TYPE_CHECKING and hasattr(_stdlib_socket.socket, "sendmsg")
+    ):
 
         @_wraps(_stdlib_socket.socket.sendmsg, assigned=(), updated=())
         async def sendmsg(self, *args):
@@ -773,7 +775,7 @@ class _SocketType(SocketType):
             # and kwargs are not accepted
             if len(args) == 4 and args[-1] is not None:
                 args = list(args)
-                args[-1] = await self._resolve_remote_address(args[-1])
+                args[-1] = await self._resolve_address_nocp(args[-1], local=False)
             return await self._nonblocking_helper(
                 _stdlib_socket.socket.sendmsg, args, {}, _core.wait_writable
             )
