@@ -6,7 +6,7 @@ async def scheduler_trace():
     trace = []
 
     async def tracer(name):
-        for i in range(10):
+        for i in range(50):
             trace.append((name, i))
             await trio.sleep(0)
 
@@ -26,9 +26,7 @@ def test_the_trio_scheduler_is_not_deterministic():
 
 
 def test_the_trio_scheduler_is_deterministic_if_seeded(monkeypatch):
-    monkeypatch.setattr(
-        trio._core._run, "_ALLOW_DETERMINISTIC_SCHEDULING", True
-    )
+    monkeypatch.setattr(trio._core._run, "_ALLOW_DETERMINISTIC_SCHEDULING", True)
     traces = []
     for _ in range(10):
         state = trio._core._run._r.getstate()
