@@ -1,14 +1,18 @@
+import os
+import re
+import sys
+
 import pytest
 
 import threading
 import socket as stdlib_socket
-import ssl as stdlib_ssl
+import ssl
 from contextlib import contextmanager
 from functools import partial
 
 from OpenSSL import SSL
 import trustme
-from async_generator import async_generator, yield_, asynccontextmanager
+from async_generator import asynccontextmanager
 
 import trio
 from .. import _core
@@ -16,8 +20,8 @@ from .._highlevel_socket import SocketStream, SocketListener
 from .._highlevel_generic import aclose_forcefully
 from .._core import ClosedResourceError, BrokenResourceError
 from .._highlevel_open_tcp_stream import open_tcp_stream
-from .. import ssl as tssl
 from .. import socket as tsocket
+from .._ssl import SSLStream, SSLListener, NeedHandshakeError, _is_eof
 from .._util import ConflictDetector
 
 from .._core.tests.tutil import slow
@@ -51,11 +55,34 @@ from ..testing import (
 TRIO_TEST_CA = trustme.CA()
 TRIO_TEST_1_CERT = TRIO_TEST_CA.issue_server_cert("trio-test-1.example.org")
 
-SERVER_CTX = stdlib_ssl.create_default_context(stdlib_ssl.Purpose.CLIENT_AUTH)
+SERVER_CTX = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
+if hasattr(ssl, "OP_IGNORE_UNEXPECTED_EOF"):
+    SERVER_CTX.options &= ~ssl.OP_IGNORE_UNEXPECTED_EOF
+
 TRIO_TEST_1_CERT.configure_cert(SERVER_CTX)
 
-CLIENT_CTX = stdlib_ssl.create_default_context()
-TRIO_TEST_CA.configure_trust(CLIENT_CTX)
+
+# TLS 1.3 has a lot of changes from previous versions. So we want to run tests
+# with both TLS 1.3, and TLS 1.2.
+# "tls13" means that we're willing to negotiate TLS 1.3. Usually that's
+# what will happen, but the renegotiation tests explicitly force a
+# downgrade on the server side. "tls12" means we refuse to negotiate TLS
+# 1.3, so we'll almost certainly use TLS 1.2.
+@pytest.fixture(scope="module", params=["tls13", "tls12"])
+def client_ctx(request):
+    ctx = ssl.create_default_context()
+
+    if hasattr(ssl, "OP_IGNORE_UNEXPECTED_EOF"):
+        ctx.options &= ~ssl.OP_IGNORE_UNEXPECTED_EOF
+
+    TRIO_TEST_CA.configure_trust(ctx)
+    if request.param in ["default", "tls13"]:
+        return ctx
+    elif request.param == "tls12":
+        ctx.maximum_version = ssl.TLSVersion.TLSv1_2
+        return ctx
+    else:  # pragma: no cover
+        assert False
 
 
 # The blocking socket server.
@@ -64,39 +91,70 @@ def ssl_echo_serve_sync(sock, *, expect_fail=False):
         wrapped = SERVER_CTX.wrap_socket(
             sock, server_side=True, suppress_ragged_eofs=False
         )
-        wrapped.do_handshake()
-        while True:
-            data = wrapped.recv(4096)
-            if not data:
-                # other side has initiated a graceful shutdown; we try to
-                # respond in kind but it's legal for them to have already gone
-                # away.
-                exceptions = (BrokenPipeError,)
-                # Under unclear conditions, CPython sometimes raises
-                # SSLWantWriteError here. This is a bug (bpo-32219), but it's
-                # not our bug, so ignore it.
-                exceptions += (stdlib_ssl.SSLWantWriteError,)
-                try:
-                    wrapped.unwrap()
-                except exceptions:
-                    pass
-                return
-            wrapped.sendall(data)
+        with wrapped:
+            wrapped.do_handshake()
+            while True:
+                data = wrapped.recv(4096)
+                if not data:
+                    # other side has initiated a graceful shutdown; we try to
+                    # respond in kind but it's legal for them to have already
+                    # gone away.
+                    exceptions = (BrokenPipeError, ssl.SSLZeroReturnError)
+                    try:
+                        wrapped.unwrap()
+                    except exceptions:
+                        pass
+                    except ssl.SSLWantWriteError:  # pragma: no cover
+                        # Under unclear conditions, CPython sometimes raises
+                        # SSLWantWriteError here. This is a bug (bpo-32219),
+                        # but it's not our bug.  Christian Heimes thinks
+                        # it's fixed in 'recent' CPython versions so we fail
+                        # the test for those and ignore it for earlier
+                        # versions.
+                        if (
+                            sys.implementation.name != "cpython"
+                            or sys.version_info >= (3, 8)
+                        ):
+                            pytest.fail(
+                                "still an issue on recent python versions "
+                                "add a comment to "
+                                "https://bugs.python.org/issue32219"
+                            )
+                    return
+                wrapped.sendall(data)
+    # This is an obscure workaround for an openssl bug. In server mode, in
+    # some versions, openssl sends some extra data at the end of do_handshake
+    # that it shouldn't send. Normally this is harmless, but, if the other
+    # side shuts down the connection before it reads that data, it might cause
+    # the OS to report a ECONNREST or even ECONNABORTED (which is just wrong,
+    # since ECONNABORTED is supposed to mean that connect() failed, but what
+    # can you do). In this case the other side did nothing wrong, but there's
+    # no way to recover, so we let it pass, and just cross our fingers its not
+    # hiding any (other) real bugs. For more details see:
+    #
+    #   https://github.com/python-trio/trio/issues/1293
+    #
+    # Also, this happens frequently but non-deterministically, so we have to
+    # 'no cover' it to avoid coverage flapping.
+    except (ConnectionResetError, ConnectionAbortedError):  # pragma: no cover
+        return
     except Exception as exc:
         if expect_fail:
             print("ssl_echo_serve_sync got error as expected:", exc)
         else:  # pragma: no cover
+            print("ssl_echo_serve_sync got unexpected error:", exc)
             raise
     else:
         if expect_fail:  # pragma: no cover
             raise RuntimeError("failed to fail?")
+    finally:
+        sock.close()
 
 
 # Fixture that gives a raw socket connected to a trio-test-1 echo server
 # (running in a thread). Useful for testing making connections with different
 # SSLContexts.
 @asynccontextmanager
-@async_generator
 async def ssl_echo_server_raw(**kwargs):
     a, b = stdlib_socket.socketpair()
     async with trio.open_nursery() as nursery:
@@ -105,24 +163,18 @@ async def ssl_echo_server_raw(**kwargs):
         # nursery context manager to exit too.
         with a, b:
             nursery.start_soon(
-                trio.run_sync_in_worker_thread,
-                partial(ssl_echo_serve_sync, b, **kwargs)
+                trio.to_thread.run_sync, partial(ssl_echo_serve_sync, b, **kwargs)
             )
 
-            await yield_(SocketStream(tsocket.from_stdlib_socket(a)))
+            yield SocketStream(tsocket.from_stdlib_socket(a))
 
 
 # Fixture that gives a properly set up SSLStream connected to a trio-test-1
 # echo server (running in a thread)
 @asynccontextmanager
-@async_generator
-async def ssl_echo_server(**kwargs):
+async def ssl_echo_server(client_ctx, **kwargs):
     async with ssl_echo_server_raw(**kwargs) as sock:
-        await yield_(
-            tssl.SSLStream(
-                sock, CLIENT_CTX, server_hostname="trio-test-1.example.org"
-            )
-        )
+        yield SSLStream(sock, client_ctx, server_hostname="trio-test-1.example.org")
 
 
 # The weird in-memory server ... thing.
@@ -133,17 +185,30 @@ class PyOpenSSLEchoStream:
         ctx = SSL.Context(SSL.SSLv23_METHOD)
         # TLS 1.3 removes renegotiation support. Which is great for them, but
         # we still have to support versions before that, and that means we
-        # need to test renegotation support, which means we need to force this
+        # need to test renegotiation support, which means we need to force this
         # to use a lower version where this test server can trigger
         # renegotiations. Of course TLS 1.3 support isn't released yet, but
         # I'm told that this will work once it is. (And once it is we can
-        # remove the pragma: no cover too.) Alternatively, once we drop
-        # support for CPython 3.5 on macOS, then we could switch to using
-        # TLSv1_2_METHOD.
+        # remove the pragma: no cover too.) Alternatively, we could switch to
+        # using TLSv1_2_METHOD.
         #
         # Discussion: https://github.com/pyca/pyopenssl/issues/624
-        if hasattr(SSL, "OP_NO_TLSv1_3"):  # pragma: no cover
-            ctx.set_options(SSL.OP_NO_TLSv1_3)
+
+        # This is the right way, but we can't use it until this PR is in a
+        # released:
+        #     https://github.com/pyca/pyopenssl/pull/861
+        #
+        # if hasattr(SSL, "OP_NO_TLSv1_3"):
+        #     ctx.set_options(SSL.OP_NO_TLSv1_3)
+        #
+        # Fortunately pyopenssl uses cryptography under the hood, so we can be
+        # confident that they're using the same version of openssl
+        from cryptography.hazmat.bindings.openssl.binding import Binding
+
+        b = Binding()
+        if hasattr(b.lib, "SSL_OP_NO_TLSv1_3"):
+            ctx.set_options(b.lib.SSL_OP_NO_TLSv1_3)
+
         # Unfortunately there's currently no way to say "use 1.3 or worse", we
         # can only disable specific versions. And if the two sides start
         # negotiating 1.4 at some point in the future, it *might* mean that
@@ -180,18 +245,20 @@ class PyOpenSSLEchoStream:
         return self._conn.renegotiate_pending()
 
     def renegotiate(self):
-        # Returns false if a renegotation is already in progress, meaning
+        # Returns false if a renegotiation is already in progress, meaning
         # nothing happens.
         assert self._conn.renegotiate()
 
     async def wait_send_all_might_not_block(self):
-        async with self._send_all_conflict_detector:
+        with self._send_all_conflict_detector:
+            await _core.checkpoint()
             await _core.checkpoint()
             await self.sleeper("wait_send_all_might_not_block")
 
     async def send_all(self, data):
         print("  --> transport_stream.send_all")
-        async with self._send_all_conflict_detector:
+        with self._send_all_conflict_detector:
+            await _core.checkpoint()
             await _core.checkpoint()
             await self.sleeper("send_all")
             self._conn.bio_write(data)
@@ -211,10 +278,13 @@ class PyOpenSSLEchoStream:
             await self.sleeper("send_all")
             print("  <-- transport_stream.send_all finished")
 
-    async def receive_some(self, nbytes):
+    async def receive_some(self, nbytes=None):
         print("  --> transport_stream.receive_some")
-        async with self._receive_some_conflict_detector:
+        if nbytes is None:
+            nbytes = 65536  # arbitrary
+        with self._receive_some_conflict_detector:
             try:
+                await _core.checkpoint()
                 await _core.checkpoint()
                 while True:
                     await self.sleeper("receive_some")
@@ -267,7 +337,7 @@ class PyOpenSSLEchoStream:
 async def test_PyOpenSSLEchoStream_gives_resource_busy_errors():
     # Make sure that PyOpenSSLEchoStream complains if two tasks call send_all
     # at the same time, or ditto for receive_some. The tricky cases where SSLStream
-    # might accidentally do this are during renegotation, which we test using
+    # might accidentally do this are during renegotiation, which we test using
     # PyOpenSSLEchoStream, so this makes sure that if we do have a bug then
     # PyOpenSSLEchoStream will notice and complain.
 
@@ -301,56 +371,47 @@ async def test_PyOpenSSLEchoStream_gives_resource_busy_errors():
 
 
 @contextmanager
-def virtual_ssl_echo_server(**kwargs):
+def virtual_ssl_echo_server(client_ctx, **kwargs):
     fakesock = PyOpenSSLEchoStream(**kwargs)
-    yield tssl.SSLStream(
-        fakesock, CLIENT_CTX, server_hostname="trio-test-1.example.org"
-    )
+    yield SSLStream(fakesock, client_ctx, server_hostname="trio-test-1.example.org")
 
 
 def ssl_wrap_pair(
-    client_transport, server_transport, *, client_kwargs={}, server_kwargs={}
+    client_ctx,
+    client_transport,
+    server_transport,
+    *,
+    client_kwargs={},
+    server_kwargs={},
 ):
-    client_ssl = tssl.SSLStream(
+    client_ssl = SSLStream(
         client_transport,
-        CLIENT_CTX,
+        client_ctx,
         server_hostname="trio-test-1.example.org",
-        **client_kwargs
+        **client_kwargs,
     )
-    server_ssl = tssl.SSLStream(
+    server_ssl = SSLStream(
         server_transport, SERVER_CTX, server_side=True, **server_kwargs
     )
     return client_ssl, server_ssl
 
 
-def ssl_memory_stream_pair(**kwargs):
+def ssl_memory_stream_pair(client_ctx, **kwargs):
     client_transport, server_transport = memory_stream_pair()
-    return ssl_wrap_pair(client_transport, server_transport, **kwargs)
+    return ssl_wrap_pair(client_ctx, client_transport, server_transport, **kwargs)
 
 
-def ssl_lockstep_stream_pair(**kwargs):
+def ssl_lockstep_stream_pair(client_ctx, **kwargs):
     client_transport, server_transport = lockstep_stream_pair()
-    return ssl_wrap_pair(client_transport, server_transport, **kwargs)
-
-
-def test_exports():
-    # Just a quick check to make sure _reexport isn't totally broken
-    assert hasattr(tssl, "SSLError")
-    assert "SSLError" in tssl.__all__
-
-    assert hasattr(tssl, "Purpose")
-    assert "Purpose" in tssl.__all__
-
-    # Intentionally omitted
-    assert not hasattr(tssl, "SSLContext")
+    return ssl_wrap_pair(client_ctx, client_transport, server_transport, **kwargs)
 
 
 # Simple smoke test for handshake/send/receive/shutdown talking to a
 # synchronous server, plus make sure that we do the bare minimum of
 # certificate checking (even though this is really Python's responsibility)
-async def test_ssl_client_basics():
+async def test_ssl_client_basics(client_ctx):
     # Everything OK
-    async with ssl_echo_server() as s:
+    async with ssl_echo_server(client_ctx) as s:
         assert not s.server_side
         await s.send_all(b"x")
         assert await s.receive_some(1) == b"x"
@@ -358,43 +419,39 @@ async def test_ssl_client_basics():
 
     # Didn't configure the CA file, should fail
     async with ssl_echo_server_raw(expect_fail=True) as sock:
-        client_ctx = stdlib_ssl.create_default_context()
-        s = tssl.SSLStream(
-            sock, client_ctx, server_hostname="trio-test-1.example.org"
-        )
+        bad_client_ctx = ssl.create_default_context()
+        s = SSLStream(sock, bad_client_ctx, server_hostname="trio-test-1.example.org")
         assert not s.server_side
         with pytest.raises(BrokenResourceError) as excinfo:
             await s.send_all(b"x")
-        assert isinstance(excinfo.value.__cause__, tssl.SSLError)
+        assert isinstance(excinfo.value.__cause__, ssl.SSLError)
 
     # Trusted CA, but wrong host name
     async with ssl_echo_server_raw(expect_fail=True) as sock:
-        s = tssl.SSLStream(
-            sock, CLIENT_CTX, server_hostname="trio-test-2.example.org"
-        )
+        s = SSLStream(sock, client_ctx, server_hostname="trio-test-2.example.org")
         assert not s.server_side
         with pytest.raises(BrokenResourceError) as excinfo:
             await s.send_all(b"x")
-        assert isinstance(excinfo.value.__cause__, tssl.CertificateError)
+        assert isinstance(excinfo.value.__cause__, ssl.CertificateError)
 
 
-async def test_ssl_server_basics():
+async def test_ssl_server_basics(client_ctx):
     a, b = stdlib_socket.socketpair()
     with a, b:
         server_sock = tsocket.from_stdlib_socket(b)
-        server_transport = tssl.SSLStream(
+        server_transport = SSLStream(
             SocketStream(server_sock), SERVER_CTX, server_side=True
         )
         assert server_transport.server_side
 
         def client():
-            client_sock = CLIENT_CTX.wrap_socket(
+            with client_ctx.wrap_socket(
                 a, server_hostname="trio-test-1.example.org"
-            )
-            client_sock.sendall(b"x")
-            assert client_sock.recv(1) == b"y"
-            client_sock.sendall(b"z")
-            client_sock.unwrap()
+            ) as client_sock:
+                client_sock.sendall(b"x")
+                assert client_sock.recv(1) == b"y"
+                client_sock.sendall(b"z")
+                client_sock.unwrap()
 
         t = threading.Thread(target=client)
         t.start()
@@ -408,13 +465,11 @@ async def test_ssl_server_basics():
         t.join()
 
 
-async def test_attributes():
+async def test_attributes(client_ctx):
     async with ssl_echo_server_raw(expect_fail=True) as sock:
-        good_ctx = CLIENT_CTX
-        bad_ctx = stdlib_ssl.create_default_context()
-        s = tssl.SSLStream(
-            sock, good_ctx, server_hostname="trio-test-1.example.org"
-        )
+        good_ctx = client_ctx
+        bad_ctx = ssl.create_default_context()
+        s = SSLStream(sock, good_ctx, server_hostname="trio-test-1.example.org")
 
         assert s.transport_stream is sock
 
@@ -444,7 +499,7 @@ async def test_attributes():
         assert s.context is bad_ctx
         with pytest.raises(BrokenResourceError) as excinfo:
             await s.do_handshake()
-        assert isinstance(excinfo.value.__cause__, tssl.SSLError)
+        assert isinstance(excinfo.value.__cause__, ssl.SSLError)
 
 
 # Note: this test fails horribly if we force TLS 1.2 and trigger a
@@ -479,7 +534,7 @@ async def test_attributes():
 # I begin to see why HTTP/2 forbids renegotiation and TLS 1.3 removes it...
 
 
-async def test_full_duplex_basics():
+async def test_full_duplex_basics(client_ctx):
     CHUNKS = 30
     CHUNK_SIZE = 32768
     EXPECTED = CHUNKS * CHUNK_SIZE
@@ -501,7 +556,7 @@ async def test_full_duplex_basics():
             chunk = await s.receive_some(CHUNK_SIZE // 2)
             received += chunk
 
-    async with ssl_echo_server() as s:
+    async with ssl_echo_server(client_ctx) as s:
         async with _core.open_nursery() as nursery:
             nursery.start_soon(sender, s)
             nursery.start_soon(receiver, s)
@@ -516,8 +571,8 @@ async def test_full_duplex_basics():
     assert sent == received
 
 
-async def test_renegotiation_simple():
-    with virtual_ssl_echo_server() as s:
+async def test_renegotiation_simple(client_ctx):
+    with virtual_ssl_echo_server(client_ctx) as s:
         await s.do_handshake()
 
         s.transport_stream.renegotiate()
@@ -535,12 +590,13 @@ async def test_renegotiation_simple():
 
 
 @slow
-async def test_renegotiation_randomized(mock_clock):
+async def test_renegotiation_randomized(mock_clock, client_ctx):
     # The only blocking things in this function are our random sleeps, so 0 is
     # a good threshold.
     mock_clock.autojump_threshold = 0
 
     import random
+
     r = random.Random(0)
 
     async def sleeper(_):
@@ -567,7 +623,7 @@ async def test_renegotiation_randomized(mock_clock):
         with assert_checkpoints():
             assert await s.receive_some(1) == expected
 
-    with virtual_ssl_echo_server(sleeper=sleeper) as s:
+    with virtual_ssl_echo_server(client_ctx, sleeper=sleeper) as s:
         await s.do_handshake()
 
         await send(b"a")
@@ -577,8 +633,8 @@ async def test_renegotiation_randomized(mock_clock):
         await clear()
 
         for i in range(100):
-            b1 = bytes([i % 0xff])
-            b2 = bytes([(2 * i) % 0xff])
+            b1 = bytes([i % 0xFF])
+            b2 = bytes([(2 * i) % 0xFF])
             s.transport_stream.renegotiate()
             async with _core.open_nursery() as nursery:
                 nursery.start_soon(send, b1)
@@ -589,8 +645,8 @@ async def test_renegotiation_randomized(mock_clock):
             await clear()
 
         for i in range(100):
-            b1 = bytes([i % 0xff])
-            b2 = bytes([(2 * i) % 0xff])
+            b1 = bytes([i % 0xFF])
+            b2 = bytes([(2 * i) % 0xFF])
             await send(b1)
             s.transport_stream.renegotiate()
             await expect(b1)
@@ -616,7 +672,7 @@ async def test_renegotiation_randomized(mock_clock):
         await trio.sleep(1000)
         await s.wait_send_all_might_not_block()
 
-    with virtual_ssl_echo_server(sleeper=sleeper_with_slow_send_all) as s:
+    with virtual_ssl_echo_server(client_ctx, sleeper=sleeper_with_slow_send_all) as s:
         await send(b"x")
         s.transport_stream.renegotiate()
         async with _core.open_nursery() as nursery:
@@ -637,7 +693,7 @@ async def test_renegotiation_randomized(mock_clock):
             await trio.sleep(1000)
 
     with virtual_ssl_echo_server(
-        sleeper=sleeper_with_slow_wait_writable_and_expect
+        client_ctx, sleeper=sleeper_with_slow_wait_writable_and_expect
     ) as s:
         await send(b"x")
         s.transport_stream.renegotiate()
@@ -650,7 +706,7 @@ async def test_renegotiation_randomized(mock_clock):
         await s.aclose()
 
 
-async def test_resource_busy_errors():
+async def test_resource_busy_errors(client_ctx):
     async def do_send_all():
         with assert_checkpoints():
             await s.send_all(b"x")
@@ -663,28 +719,28 @@ async def test_resource_busy_errors():
         with assert_checkpoints():
             await s.wait_send_all_might_not_block()
 
-    s, _ = ssl_lockstep_stream_pair()
+    s, _ = ssl_lockstep_stream_pair(client_ctx)
     with pytest.raises(_core.BusyResourceError) as excinfo:
         async with _core.open_nursery() as nursery:
             nursery.start_soon(do_send_all)
             nursery.start_soon(do_send_all)
     assert "another task" in str(excinfo.value)
 
-    s, _ = ssl_lockstep_stream_pair()
+    s, _ = ssl_lockstep_stream_pair(client_ctx)
     with pytest.raises(_core.BusyResourceError) as excinfo:
         async with _core.open_nursery() as nursery:
             nursery.start_soon(do_receive_some)
             nursery.start_soon(do_receive_some)
     assert "another task" in str(excinfo.value)
 
-    s, _ = ssl_lockstep_stream_pair()
+    s, _ = ssl_lockstep_stream_pair(client_ctx)
     with pytest.raises(_core.BusyResourceError) as excinfo:
         async with _core.open_nursery() as nursery:
             nursery.start_soon(do_send_all)
             nursery.start_soon(do_wait_send_all_might_not_block)
     assert "another task" in str(excinfo.value)
 
-    s, _ = ssl_lockstep_stream_pair()
+    s, _ = ssl_lockstep_stream_pair(client_ctx)
     with pytest.raises(_core.BusyResourceError) as excinfo:
         async with _core.open_nursery() as nursery:
             nursery.start_soon(do_wait_send_all_might_not_block)
@@ -699,14 +755,18 @@ async def test_wait_writable_calls_underlying_wait_writable():
         async def wait_send_all_might_not_block(self):
             record.append("ok")
 
-    ctx = stdlib_ssl.create_default_context()
-    s = tssl.SSLStream(NotAStream(), ctx, server_hostname="x")
+    ctx = ssl.create_default_context()
+    s = SSLStream(NotAStream(), ctx, server_hostname="x")
     await s.wait_send_all_might_not_block()
     assert record == ["ok"]
 
 
-async def test_checkpoints():
-    async with ssl_echo_server() as s:
+@pytest.mark.skipif(
+    os.name == "nt" and sys.version_info >= (3, 10),
+    reason="frequently fails on Windows + Python 3.10",
+)
+async def test_checkpoints(client_ctx):
+    async with ssl_echo_server(client_ctx) as s:
         with assert_checkpoints():
             await s.do_handshake()
         with assert_checkpoints():
@@ -728,14 +788,14 @@ async def test_checkpoints():
         with assert_checkpoints():
             await s.unwrap()
 
-    async with ssl_echo_server() as s:
+    async with ssl_echo_server(client_ctx) as s:
         await s.do_handshake()
         with assert_checkpoints():
             await s.aclose()
 
 
-async def test_send_all_empty_string():
-    async with ssl_echo_server() as s:
+async def test_send_all_empty_string(client_ctx):
+    async with ssl_echo_server(client_ctx) as s:
         await s.do_handshake()
 
         # underlying SSLObject interprets writing b"" as indicating an EOF,
@@ -751,15 +811,16 @@ async def test_send_all_empty_string():
 
 
 @pytest.mark.parametrize("https_compatible", [False, True])
-async def test_SSLStream_generic(https_compatible):
+async def test_SSLStream_generic(client_ctx, https_compatible):
     async def stream_maker():
         return ssl_memory_stream_pair(
+            client_ctx,
             client_kwargs={"https_compatible": https_compatible},
             server_kwargs={"https_compatible": https_compatible},
         )
 
     async def clogged_stream_maker():
-        client, server = ssl_lockstep_stream_pair()
+        client, server = ssl_lockstep_stream_pair(client_ctx)
         # If we don't do handshakes up front, then we run into a problem in
         # the following situation:
         # - server does wait_send_all_might_not_block
@@ -774,8 +835,8 @@ async def test_SSLStream_generic(https_compatible):
     await check_two_way_stream(stream_maker, clogged_stream_maker)
 
 
-async def test_unwrap():
-    client_ssl, server_ssl = ssl_memory_stream_pair()
+async def test_unwrap(client_ctx):
+    client_ssl, server_ssl = ssl_memory_stream_pair(client_ctx)
     client_transport = client_ssl.transport_stream
     server_transport = server_ssl.transport_stream
 
@@ -828,10 +889,10 @@ async def test_unwrap():
         nursery.start_soon(server)
 
 
-async def test_closing_nice_case():
+async def test_closing_nice_case(client_ctx):
     # the nice case: graceful closes all around
 
-    client_ssl, server_ssl = ssl_memory_stream_pair()
+    client_ssl, server_ssl = ssl_memory_stream_pair(client_ctx)
     client_transport = client_ssl.transport_stream
 
     # Both the handshake and the close require back-and-forth discussion, so
@@ -861,27 +922,23 @@ async def test_closing_nice_case():
         await client_ssl.aclose()
 
     # Trying to send more data does not work
-    with assert_checkpoints():
-        with pytest.raises(ClosedResourceError):
-            await server_ssl.send_all(b"123")
+    with pytest.raises(ClosedResourceError):
+        await server_ssl.send_all(b"123")
 
     # And once the connection is has been closed *locally*, then instead of
     # getting empty bytestrings we get a proper error
-    with assert_checkpoints():
-        with pytest.raises(ClosedResourceError):
-            await client_ssl.receive_some(10) == b""
+    with pytest.raises(ClosedResourceError):
+        await client_ssl.receive_some(10) == b""
 
-    with assert_checkpoints():
-        with pytest.raises(ClosedResourceError):
-            await client_ssl.unwrap()
+    with pytest.raises(ClosedResourceError):
+        await client_ssl.unwrap()
 
-    with assert_checkpoints():
-        with pytest.raises(ClosedResourceError):
-            await client_ssl.do_handshake()
+    with pytest.raises(ClosedResourceError):
+        await client_ssl.do_handshake()
 
     # Check that a graceful close *before* handshaking gives a clean EOF on
     # the other side
-    client_ssl, server_ssl = ssl_memory_stream_pair()
+    client_ssl, server_ssl = ssl_memory_stream_pair(client_ctx)
 
     async def expect_eof_server():
         with assert_checkpoints():
@@ -894,8 +951,8 @@ async def test_closing_nice_case():
         nursery.start_soon(expect_eof_server)
 
 
-async def test_send_all_fails_in_the_middle():
-    client, server = ssl_memory_stream_pair()
+async def test_send_all_fails_in_the_middle(client_ctx):
+    client, server = ssl_memory_stream_pair(client_ctx)
 
     async with _core.open_nursery() as nursery:
         nursery.start_soon(client.do_handshake)
@@ -925,18 +982,18 @@ async def test_send_all_fails_in_the_middle():
     assert closed == 2
 
 
-async def test_ssl_over_ssl():
+async def test_ssl_over_ssl(client_ctx):
     client_0, server_0 = memory_stream_pair()
 
-    client_1 = tssl.SSLStream(
-        client_0, CLIENT_CTX, server_hostname="trio-test-1.example.org"
+    client_1 = SSLStream(
+        client_0, client_ctx, server_hostname="trio-test-1.example.org"
     )
-    server_1 = tssl.SSLStream(server_0, SERVER_CTX, server_side=True)
+    server_1 = SSLStream(server_0, SERVER_CTX, server_side=True)
 
-    client_2 = tssl.SSLStream(
-        client_1, CLIENT_CTX, server_hostname="trio-test-1.example.org"
+    client_2 = SSLStream(
+        client_1, client_ctx, server_hostname="trio-test-1.example.org"
     )
-    server_2 = tssl.SSLStream(server_1, SERVER_CTX, server_side=True)
+    server_2 = SSLStream(server_1, SERVER_CTX, server_side=True)
 
     async def client():
         await client_2.send_all(b"hi")
@@ -951,8 +1008,8 @@ async def test_ssl_over_ssl():
         nursery.start_soon(server)
 
 
-async def test_ssl_bad_shutdown():
-    client, server = ssl_memory_stream_pair()
+async def test_ssl_bad_shutdown(client_ctx):
+    client, server = ssl_memory_stream_pair(client_ctx)
 
     async with _core.open_nursery() as nursery:
         nursery.start_soon(client.do_handshake)
@@ -968,10 +1025,11 @@ async def test_ssl_bad_shutdown():
     await server.aclose()
 
 
-async def test_ssl_bad_shutdown_but_its_ok():
+async def test_ssl_bad_shutdown_but_its_ok(client_ctx):
     client, server = ssl_memory_stream_pair(
+        client_ctx,
         server_kwargs={"https_compatible": True},
-        client_kwargs={"https_compatible": True}
+        client_kwargs={"https_compatible": True},
     )
 
     async with _core.open_nursery() as nursery:
@@ -994,10 +1052,8 @@ async def test_ssl_handshake_failure_during_aclose():
     # the underlying transport.
     async with ssl_echo_server_raw(expect_fail=True) as sock:
         # Don't configure trust correctly
-        client_ctx = stdlib_ssl.create_default_context()
-        s = tssl.SSLStream(
-            sock, client_ctx, server_hostname="trio-test-1.example.org"
-        )
+        client_ctx = ssl.create_default_context()
+        s = SSLStream(sock, client_ctx, server_hostname="trio-test-1.example.org")
         # It's a little unclear here whether aclose should swallow the error
         # or let it escape. We *do* swallow the error if it arrives when we're
         # sending close_notify, because both sides closing the connection
@@ -1008,10 +1064,10 @@ async def test_ssl_handshake_failure_during_aclose():
             await s.aclose()
 
 
-async def test_ssl_only_closes_stream_once():
+async def test_ssl_only_closes_stream_once(client_ctx):
     # We used to have a bug where if transport_stream.aclose() raised an
     # error, we would call it again. This checks that that's fixed.
-    client, server = ssl_memory_stream_pair()
+    client, server = ssl_memory_stream_pair(client_ctx)
 
     async with _core.open_nursery() as nursery:
         nursery.start_soon(client.do_handshake)
@@ -1033,10 +1089,11 @@ async def test_ssl_only_closes_stream_once():
     assert transport_close_count == 1
 
 
-async def test_ssl_https_compatibility_disagreement():
+async def test_ssl_https_compatibility_disagreement(client_ctx):
     client, server = ssl_memory_stream_pair(
+        client_ctx,
         server_kwargs={"https_compatible": False},
-        client_kwargs={"https_compatible": True}
+        client_kwargs={"https_compatible": True},
     )
 
     async with _core.open_nursery() as nursery:
@@ -1048,17 +1105,19 @@ async def test_ssl_https_compatibility_disagreement():
     async def receive_and_expect_error():
         with pytest.raises(BrokenResourceError) as excinfo:
             await server.receive_some(10)
-        assert isinstance(excinfo.value.__cause__, tssl.SSLEOFError)
+
+        assert _is_eof(excinfo.value.__cause__)
 
     async with _core.open_nursery() as nursery:
         nursery.start_soon(client.aclose)
         nursery.start_soon(receive_and_expect_error)
 
 
-async def test_https_mode_eof_before_handshake():
+async def test_https_mode_eof_before_handshake(client_ctx):
     client, server = ssl_memory_stream_pair(
+        client_ctx,
         server_kwargs={"https_compatible": True},
-        client_kwargs={"https_compatible": True}
+        client_kwargs={"https_compatible": True},
     )
 
     async def server_expect_clean_eof():
@@ -1069,8 +1128,8 @@ async def test_https_mode_eof_before_handshake():
         nursery.start_soon(server_expect_clean_eof)
 
 
-async def test_send_error_during_handshake():
-    client, server = ssl_memory_stream_pair()
+async def test_send_error_during_handshake(client_ctx):
+    client, server = ssl_memory_stream_pair(client_ctx)
 
     async def bad_hook():
         raise KeyError
@@ -1086,8 +1145,8 @@ async def test_send_error_during_handshake():
             await client.do_handshake()
 
 
-async def test_receive_error_during_handshake():
-    client, server = ssl_memory_stream_pair()
+async def test_receive_error_during_handshake(client_ctx):
+    client, server = ssl_memory_stream_pair(client_ctx)
 
     async def bad_hook():
         raise KeyError
@@ -1109,9 +1168,86 @@ async def test_receive_error_during_handshake():
             await client.do_handshake()
 
 
-async def test_getpeercert():
+async def test_selected_alpn_protocol_before_handshake(client_ctx):
+    client, server = ssl_memory_stream_pair(client_ctx)
+
+    with pytest.raises(NeedHandshakeError):
+        client.selected_alpn_protocol()
+
+    with pytest.raises(NeedHandshakeError):
+        server.selected_alpn_protocol()
+
+
+async def test_selected_alpn_protocol_when_not_set(client_ctx):
+    # ALPN protocol still returns None when it's not set,
+    # instead of raising an exception
+    client, server = ssl_memory_stream_pair(client_ctx)
+
+    async with _core.open_nursery() as nursery:
+        nursery.start_soon(client.do_handshake)
+        nursery.start_soon(server.do_handshake)
+
+    assert client.selected_alpn_protocol() is None
+    assert server.selected_alpn_protocol() is None
+
+    assert client.selected_alpn_protocol() == server.selected_alpn_protocol()
+
+
+async def test_selected_npn_protocol_before_handshake(client_ctx):
+    client, server = ssl_memory_stream_pair(client_ctx)
+
+    with pytest.raises(NeedHandshakeError):
+        client.selected_npn_protocol()
+
+    with pytest.raises(NeedHandshakeError):
+        server.selected_npn_protocol()
+
+
+@pytest.mark.filterwarnings(
+    r"ignore: ssl module. NPN is deprecated, use ALPN instead:UserWarning",
+    r"ignore:ssl NPN is deprecated, use ALPN instead:DeprecationWarning",
+)
+async def test_selected_npn_protocol_when_not_set(client_ctx):
+    # NPN protocol still returns None when it's not set,
+    # instead of raising an exception
+    client, server = ssl_memory_stream_pair(client_ctx)
+
+    async with _core.open_nursery() as nursery:
+        nursery.start_soon(client.do_handshake)
+        nursery.start_soon(server.do_handshake)
+
+    assert client.selected_npn_protocol() is None
+    assert server.selected_npn_protocol() is None
+
+    assert client.selected_npn_protocol() == server.selected_npn_protocol()
+
+
+async def test_get_channel_binding_before_handshake(client_ctx):
+    client, server = ssl_memory_stream_pair(client_ctx)
+
+    with pytest.raises(NeedHandshakeError):
+        client.get_channel_binding()
+
+    with pytest.raises(NeedHandshakeError):
+        server.get_channel_binding()
+
+
+async def test_get_channel_binding_after_handshake(client_ctx):
+    client, server = ssl_memory_stream_pair(client_ctx)
+
+    async with _core.open_nursery() as nursery:
+        nursery.start_soon(client.do_handshake)
+        nursery.start_soon(server.do_handshake)
+
+    assert client.get_channel_binding() is not None
+    assert server.get_channel_binding() is not None
+
+    assert client.get_channel_binding() == server.get_channel_binding()
+
+
+async def test_getpeercert(client_ctx):
     # Make sure we're not affected by https://bugs.python.org/issue29334
-    client, server = ssl_memory_stream_pair()
+    client, server = ssl_memory_stream_pair(client_ctx)
 
     async with _core.open_nursery() as nursery:
         nursery.start_soon(client.do_handshake)
@@ -1119,25 +1255,20 @@ async def test_getpeercert():
 
     assert server.getpeercert() is None
     print(client.getpeercert())
-    assert (
-        ("DNS",
-         "trio-test-1.example.org") in client.getpeercert()["subjectAltName"]
-    )
+    assert ("DNS", "trio-test-1.example.org") in client.getpeercert()["subjectAltName"]
 
 
-async def test_SSLListener():
+async def test_SSLListener(client_ctx):
     async def setup(**kwargs):
         listen_sock = tsocket.socket()
         await listen_sock.bind(("127.0.0.1", 0))
         listen_sock.listen(1)
         socket_listener = SocketListener(listen_sock)
-        ssl_listener = tssl.SSLListener(socket_listener, SERVER_CTX, **kwargs)
+        ssl_listener = SSLListener(socket_listener, SERVER_CTX, **kwargs)
 
         transport_client = await open_tcp_stream(*listen_sock.getsockname())
-        ssl_client = tssl.SSLStream(
-            transport_client,
-            CLIENT_CTX,
-            server_hostname="trio-test-1.example.org"
+        ssl_client = SSLStream(
+            transport_client, client_ctx, server_hostname="trio-test-1.example.org"
         )
         return listen_sock, ssl_listener, ssl_client
 
@@ -1160,16 +1291,12 @@ async def test_SSLListener():
 
     ################
 
-    # Test https_compatible and max_refill_bytes
-    _, ssl_listener, ssl_client = await setup(
-        https_compatible=True,
-        max_refill_bytes=100,
-    )
+    # Test https_compatible
+    _, ssl_listener, ssl_client = await setup(https_compatible=True)
 
     ssl_server = await ssl_listener.accept()
 
     assert ssl_server._https_compatible
-    assert ssl_server._max_refill_bytes == 100
 
     await aclose_forcefully(ssl_listener)
     await aclose_forcefully(ssl_client)
